@@ -1,129 +1,56 @@
-import json
 from collections import defaultdict
-from typing import List, Optional
-from urllib.parse import quote_plus
+from dataclasses import asdict, dataclass
+from enum import Enum
+from typing import List
 
-import requests
-
-from molgenis.bbmri_eric import utils
 from molgenis.bbmri_eric.model import (
+    EricData,
     ExternalServerNode,
+    MixedData,
     Node,
     NodeData,
+    OntologyTable,
     QualityInfo,
     Source,
     Table,
     TableMeta,
     TableType,
 )
-from molgenis.bbmri_eric.utils import batched
-from molgenis.client import Session
+from molgenis.client import MolgenisRequestError, Session
 
 
-class ExtendedSession(Session):
-    """
-    Class containing functionality that the base molgenis python client Session class
-    does not have. Methods in this class could be moved to molgenis-py-client someday.
-    """
-
-    def __init__(self, url: str, token: Optional[str] = None):
-        super(ExtendedSession, self).__init__(url, token)
-        self.url = self._root_url
-
-    def get_uploadable_data(self, entity_type_id: str, *args, **kwargs) -> List[dict]:
-        """
-        Returns all the rows of an entity type, transformed to the uploadable format.
-        """
-        rows = self.get(entity_type_id, *args, **kwargs)
-        return utils.to_upload_format(rows)
-
-    def upsert_batched(self, entity_type_id: str, entities: List[dict]):
-        """
-        Upserts entities in an entity type (in batches, if needed).
-        @param entity_type_id: the id of the entity type to upsert to
-        @param entities: the entities to upsert
-        """
-        # Get the existing identifiers
-        meta = self.get_meta(entity_type_id)
-        id_attr = meta.id_attribute
-        existing_entities = self.get(meta.id, batch_size=10000, attributes=id_attr)
-        existing_ids = {entity[id_attr] for entity in existing_entities}
-
-        # Based on the existing identifiers, decide which rows should be added/updated
-        add = list()
-        update = list()
-        for entity in entities:
-            if entity[id_attr] in existing_ids:
-                update.append(entity)
-            else:
-                add.append(entity)
-
-        # Sanitize data: rows that are added should not contain one_to_manys
-        add = utils.remove_one_to_manys(add, meta)
-
-        # Do the adds and updates in batches
-        self.add_batched(meta.id, meta.self_references, add)
-        self.update_batched(meta.id, meta.self_references, update)
-
-    def update(self, entity_type_id: str, entities: List[dict]):
-        """Updates multiple entities."""
-        response = self._session.put(
-            self._api_url + "v2/" + quote_plus(entity_type_id),
-            headers=self._get_token_header_with_content_type(),
-            data=json.dumps({"entities": entities}),
-        )
-
-        try:
-            response.raise_for_status()
-        except requests.RequestException as ex:
-            self._raise_exception(ex)
-
-        return response
-
-    def update_batched(
-        self, entity_type_id: str, self_references: List[str], entities: List[dict]
-    ):
-        """Updates multiple entities in batches of 1000."""
-        # TODO updating things in bulk will fail if there are self-references across
-        #  batches. Dependency resolving is needed.
-        #  2022-03: Partly solved by the next two lines. However this might not be
-        #  enough for more complex cases.
-        if self_references and len(entities) > 1000:
-            entities = utils.sort_self_references(entities, self_references)
-        batches = list(batched(entities, 1000))
-        for batch in batches:
-            self.update(entity_type_id, batch)
-
-    def add_batched(
-        self, entity_type_id: str, self_references: List[str], entities: List[dict]
-    ):
-        """Adds multiple entities in batches of 1000."""
-        # TODO adding things in bulk will fail if there are self-references across
-        #  batches. Dependency resolving is needed.
-        #  2022-03: Partly solved by the next two lines. However this might not be
-        #  enough for more complex cases.
-        if self_references and len(entities) > 1000:
-            entities = utils.sort_self_references(entities, self_references)
-        batches = list(batched(entities, 1000))
-        for batch in batches:
-            self.add_all(entity_type_id, batch)
-
-    def get_meta(self, entity_type_id: str) -> TableMeta:
-        """Similar to get_entity_meta_data() of the parent Session class, but uses the
-        newer Metadata API instead of the REST API V1."""
-        response = self._session.get(
-            self._api_url + "metadata/" + quote_plus(entity_type_id),
-            headers=self._get_token_header(),
-        )
-        try:
-            response.raise_for_status()
-        except requests.RequestException as ex:
-            self._raise_exception(ex)
-
-        return TableMeta(meta=response.json())
+@dataclass
+class AttributesRequest:
+    persons: List[str]
+    networks: List[str]
+    also_known_in: List[str]
+    biobanks: List[str]
+    collections: List[str]
 
 
-class EricSession(ExtendedSession):
+class MolgenisImportError(MolgenisRequestError):
+    pass
+
+
+class ImportDataAction(Enum):
+    """Enum of MOLGENIS import actions"""
+
+    ADD = "add"
+    ADD_UPDATE_EXISTING = "add_update_existing"
+    UPDATE = "update"
+    ADD_IGNORE_EXISTING = "add_ignore_existing"
+
+
+class ImportMetadataAction(Enum):
+    """Enum of MOLGENIS import metadata actions"""
+
+    ADD = "add"
+    UPDATE = "update"
+    UPSERT = "upsert"
+    IGNORE = "ignore"
+
+
+class EricSession(Session):
     """
     A session with a BBMRI ERIC directory. Contains methods to get national nodes,
     their (staging) data and quality information.
@@ -134,6 +61,26 @@ class EricSession(ExtendedSession):
 
     NODES_TABLE = "eu_bbmri_eric_national_nodes"
 
+    def get_ontology(
+        self,
+        entity_type_id: str,
+        parent_attr: str = "parentId",
+    ) -> OntologyTable:
+        """
+        Retrieves an ontology table.
+        :param entity_type_id: the identifier of the table
+        :param parent_attr: the name of the attribute that contains the parent relation
+        :return: an OntologyTable
+        """
+        rows = self.get(
+            entity_type_id,
+            batch_size=10000,
+            attributes=f"id,{parent_attr},ontology",
+            uploadable=True,
+        )
+        meta = TableMeta(meta=self.get_meta(entity_type_id))
+        return OntologyTable.of(meta, rows, parent_attr)
+
     def get_quality_info(self) -> QualityInfo:
         """
         Retrieves the quality information identifiers for biobanks and collections.
@@ -141,23 +88,35 @@ class EricSession(ExtendedSession):
         """
 
         biobank_qualities = self.get(
-            "eu_bbmri_eric_bio_qual_info", batch_size=10000, attributes="id,biobank"
+            "eu_bbmri_eric_bio_qual_info",
+            batch_size=10000,
+            attributes="id,biobank,assess_level_bio",
+            uploadable=True,
         )
         collection_qualities = self.get(
-            "eu_bbmri_eric_col_qual_info", batch_size=10000, attributes="id,collection"
+            "eu_bbmri_eric_col_qual_info",
+            batch_size=10000,
+            attributes="id,collection,assess_level_col",
+            uploadable=True,
         )
 
-        biobanks = utils.to_upload_format(biobank_qualities)
-        collections = utils.to_upload_format(collection_qualities)
-
         bb_qual = defaultdict(list)
+        bb_level = defaultdict(list)
         coll_qual = defaultdict(list)
-        for row in biobanks:
+        coll_level = defaultdict(list)
+        for row in biobank_qualities:
             bb_qual[row["biobank"]].append(row["id"])
-        for row in collections:
+            bb_level[row["biobank"]].append(row["assess_level_bio"])
+        for row in collection_qualities:
             coll_qual[row["collection"]].append(row["id"])
+            coll_level[row["collection"]].append(row["assess_level_col"])
 
-        return QualityInfo(biobanks=bb_qual, collections=coll_qual)
+        return QualityInfo(
+            biobanks=bb_qual,
+            collections=coll_qual,
+            biobank_levels=bb_level,
+            collection_levels=coll_level,
+        )
 
     def get_node(self, code: str) -> Node:
         """
@@ -238,7 +197,7 @@ class EricSession(ExtendedSession):
 
     def get_staging_node_data(self, node: Node) -> NodeData:
         """
-        Gets the four tables that belong to a single node's staging area.
+        Gets the five tables that belong to a single node's staging area.
 
         :param Node node: the node to get the staging data for
         :return: a NodeData object
@@ -246,19 +205,19 @@ class EricSession(ExtendedSession):
         tables = dict()
         for table_type in TableType.get_import_order():
             id_ = node.get_staging_id(table_type)
-            meta = self.get_meta(id_)
+            meta = TableMeta(meta=self.get_meta(id_))
 
-            tables[table_type] = Table.of(
+            tables[table_type.value] = Table.of(
                 table_type=table_type,
                 meta=meta,
-                rows=self.get_uploadable_data(id_, batch_size=10000),
+                rows=self.get(id_, batch_size=10000, uploadable=True),
             )
 
         return NodeData.from_dict(node=node, source=Source.STAGING, tables=tables)
 
     def get_published_node_data(self, node: Node) -> NodeData:
         """
-        Gets the four tables that belong to a single node from the published tables.
+        Gets the five tables that belong to a single node from the published tables.
         Filters the rows based on the national_node field.
 
         :param Node node: the node to get the published data for
@@ -268,20 +227,77 @@ class EricSession(ExtendedSession):
         tables = dict()
         for table_type in TableType.get_import_order():
             id_ = table_type.base_id
-            meta = self.get_meta(id_)
+            meta = TableMeta(self.get_meta(id_))
 
-            tables[table_type] = Table.of(
+            tables[table_type.value] = Table.of(
                 table_type=table_type,
                 meta=meta,
-                rows=self.get_uploadable_data(
-                    id_, batch_size=10000, q=f"national_node=={node.code}"
+                rows=self.get(
+                    id_,
+                    batch_size=10000,
+                    q=f"national_node=={node.code}",
+                    uploadable=True,
                 ),
             )
 
         return NodeData.from_dict(node=node, source=Source.PUBLISHED, tables=tables)
 
+    def get_published_data(
+        self, nodes: List[Node], attributes: AttributesRequest
+    ) -> MixedData:
+        """
+        Gets the five tables that belong to one or more nodes from the published tables.
+        Filters the rows based on the national_node field.
 
-class ExternalServerSession(ExtendedSession):
+        :param List[Node] nodes: the node(s) to get the published data for
+        :param AttributesRequest attributes: the attributes to get for each table
+        :return: an EricData object
+        """
+
+        if len(nodes) == 0:
+            raise ValueError("No nodes provided")
+
+        attributes = asdict(attributes)
+        codes = [node.code for node in nodes]
+        tables = dict()
+        for table_type in TableType.get_import_order():
+            id_ = table_type.base_id
+            meta = TableMeta(self.get_meta(id_))
+            attrs = attributes[table_type.value]
+
+            tables[table_type.value] = Table.of(
+                table_type=table_type,
+                meta=meta,
+                rows=self.get(
+                    id_,
+                    batch_size=10000,
+                    q=f"national_node=in=({','.join(codes)})",
+                    attributes=",".join(attrs),
+                    uploadable=True,
+                ),
+            )
+
+        return MixedData.from_mixed_dict(source=Source.PUBLISHED, tables=tables)
+
+    def upload_data(self, data: EricData):
+        """
+        Converts the five tables of an EricData object to CSV, bundles them in
+        a ZIP archive and imports them through the import API.
+        :param data: an EricData object
+        """
+
+        importable_data = dict()
+        for table in data.import_order:
+            importable_data[table.full_name] = table.rows
+
+        self.import_data(
+            importable_data,
+            data_action=ImportDataAction.ADD_UPDATE_EXISTING,
+            metadata_action=ImportMetadataAction.IGNORE,
+        )
+
+
+class ExternalServerSession(Session):
     """
     A session with a national node's external server (for example BBMRI-NL).
     """
@@ -292,7 +308,7 @@ class ExternalServerSession(ExtendedSession):
 
     def get_node_data(self) -> NodeData:
         """
-        Gets the four tables of this node's external server.
+        Gets the five tables of this node's external server.
 
         :return: a NodeData object
         """
@@ -300,13 +316,15 @@ class ExternalServerSession(ExtendedSession):
         tables = dict()
         for table_type in TableType.get_import_order():
             id_ = table_type.base_id
-            meta = self.get_meta(id_)
-
-            tables[table_type] = Table.of(
-                table_type=table_type,
-                meta=meta,
-                rows=self.get_uploadable_data(id_, batch_size=10000),
-            )
+            if not self.get("sys_md_EntityType", q=f"id=={id_}"):
+                tables[table_type.value] = Table.of_placeholder(table_type)
+            else:
+                meta = TableMeta(self.get_meta(id_))
+                tables[table_type.value] = Table.of(
+                    table_type=table_type,
+                    meta=meta,
+                    rows=self.get(id_, batch_size=10000, uploadable=True),
+                )
 
         return NodeData.from_dict(
             node=self.node, source=Source.EXTERNAL_SERVER, tables=tables
