@@ -1,5 +1,6 @@
+from molgenis.bbmri_eric.categories import CategoryMapper
 from molgenis.bbmri_eric.errors import EricWarning
-from molgenis.bbmri_eric.model import Node, NodeData, QualityInfo, Table
+from molgenis.bbmri_eric.model import Node, NodeData, OntologyTable, QualityInfo, Table
 from molgenis.bbmri_eric.printer import Printer
 
 
@@ -17,15 +18,18 @@ class Transformer:
         printer: Printer,
         existing_biobanks: Table,
         eu_node_data: NodeData,
+        diseases: OntologyTable,
     ):
         self.node_data = node_data
         self.quality = quality
         self.printer = printer
         self.existing_biobanks = existing_biobanks.rows_by_id
         self.eu_node_data = eu_node_data
+        self.category_mapper = CategoryMapper(diseases)
+
         self.warnings = []
 
-    def enrich(self):
+    def transform(self):
         """
         Transforms the data of a node:
         1. Sets the commercial use boolean
@@ -34,13 +38,17 @@ class Transformer:
         4. Adds PIDs to biobanks
         5. Replaces a node's EU rows with the data from node EU's staging area
         6. Adds the combined networks' ids to collections
+        7. Merges biobank 'covid19biobank' values into 'capabilities'
         """
         self._set_national_node_code()
         self._replace_eu_rows()
         self._set_commercial_use_bool()
         self._set_quality_info()
         self._set_biobank_pids()
+        self._set_biobank_labels()
         self._set_combined_networks()
+        self._set_combined_qualities()
+        self._set_collection_categories()
         return self.warnings
 
     def _set_commercial_use_bool(self):
@@ -130,6 +138,16 @@ class Transformer:
                     self.printer.print_warning(warning, indent=1)
                     self.warnings.append(warning)
 
+    def _set_biobank_labels(self):
+        """
+        For every collection of the Node, adds the `biobank_label` field, with the
+        name of its biobank
+        """
+        self.printer.print("Adding biobank labels")
+        for collection in self.node_data.collections.rows:
+            biobank = self.node_data.biobanks.rows_by_id[collection["biobank"]]
+            collection["biobank_label"] = biobank["name"]
+
     def _set_combined_networks(self):
         """
         For every collection of the Node, adds to the `combined_network` field, the
@@ -141,3 +159,26 @@ class Transformer:
             collection["combined_network"] = list(
                 set(biobank["network"] + collection["network"])
             )
+
+    def _set_combined_qualities(self):
+        """
+        For every collection of the Node, adds to the `combined_quality` field, the
+        union of the qualities of the collection itself and the ones of its biobank
+        """
+        self.printer.print("Adding combined qualities")
+
+        bb_levels = self.quality.get_levels(self.node_data.biobanks.type)
+        coll_levels = self.quality.get_levels(self.node_data.collections.type)
+        for collection in self.node_data.collections.rows:
+            biobank = self.node_data.biobanks.rows_by_id[collection["biobank"]]
+            bb_level = bb_levels.get(biobank["id"], [])
+            coll_level = coll_levels.get(collection["id"], [])
+            collection["combined_quality"] = list(set(bb_level + coll_level))
+
+    def _set_collection_categories(self):
+        """
+        Sets the 'categories' field of each collection based on the collection values.
+        """
+        self.printer.print("Setting collection categories")
+        for collection in self.node_data.collections.rows:
+            collection["categories"] = self.category_mapper.map(collection)
