@@ -1,28 +1,36 @@
-from unittest.mock import MagicMock
+from unittest.mock import MagicMock, call
+
+import pytest
 
 from molgenis.bbmri_eric.errors import EricWarning
 from molgenis.bbmri_eric.model import Node, QualityInfo, Table, TableType
-from molgenis.bbmri_eric.printer import Printer
 from molgenis.bbmri_eric.transformer import Transformer
 
 
-def test_transformer_node_codes(node_data):
+@pytest.fixture
+def transformer():
+    return Transformer(
+        node_data=MagicMock(),
+        quality=MagicMock(),
+        printer=MagicMock(),
+        existing_biobanks=MagicMock(),
+        eu_node_data=MagicMock(),
+        diseases=MagicMock(),
+    )
+
+
+def test_transformer_node_codes(node_data, transformer):
     for table in node_data.import_order:
         assert "national_node" not in table.rows[0]
 
-    Transformer(
-        node_data=node_data,
-        quality=MagicMock(),
-        printer=Printer(),
-        existing_biobanks=MagicMock(),
-        eu_node_data=MagicMock(),
-    )._set_national_node_code()
+    transformer.node_data = node_data
+    transformer._set_national_node_code()
 
     for table in node_data.import_order:
-        assert table.rows[0]["national_node"] == "NO"
+        assert table.rows[0]["national_node"] == "NL"
 
 
-def test_transformer_commercial_use():
+def test_transformer_commercial_use(transformer):
     node_data = MagicMock()
     node_data.collections.rows = [
         {"biobank": "biobank1", "collaboration_commercial": True},
@@ -41,13 +49,8 @@ def test_transformer_commercial_use():
         "biobank3": {"collaboration_commercial": False},
     }
 
-    Transformer(
-        node_data=node_data,
-        quality=MagicMock(),
-        printer=Printer(),
-        existing_biobanks=MagicMock(),
-        eu_node_data=MagicMock(),
-    )._set_commercial_use_bool()
+    transformer.node_data = node_data
+    transformer._set_commercial_use_bool()
 
     assert node_data.collections.rows[0]["commercial_use"] is True
     assert node_data.collections.rows[1]["commercial_use"] is False
@@ -60,58 +63,47 @@ def test_transformer_commercial_use():
     assert node_data.collections.rows[8]["commercial_use"] is False
 
 
-def test_transformer_quality(node_data):
+def test_transformer_quality(node_data, transformer):
     q_info = QualityInfo(
         biobanks={
-            "bbmri-eric:ID:NO_BIOBANK1": ["quality1", "quality2"],
-            "bbmri-eric:ID:NO_CoronaTrondelag": ["quality3"],
-        },
-        collections={
-            "bbmri-eric:ID:NO_bbmri-eric:ID:NO_CancerBiobankOUH:collection"
-            ":all_samples_samples": ["quality1"]
+            "bbmri-eric:ID:NL_test_quality_biobank1": ["quality1", "quality2"],
+            "bbmri-eric:ID:NL_test_quality_biobank2": ["quality3"],
         },
+        biobank_levels={},
+        collections={"bbmri-eric:ID:NL_bb1:collection:test_quality1": ["quality1"]},
+        collection_levels={},
     )
+    transformer.node_data = node_data
+    transformer.quality = q_info
 
-    Transformer(
-        node_data=node_data,
-        quality=q_info,
-        printer=Printer(),
-        existing_biobanks=MagicMock(),
-        eu_node_data=MagicMock(),
-    )._set_quality_info()
+    transformer._set_quality_info()
 
-    assert node_data.biobanks.rows_by_id["bbmri-eric:ID:NO_BIOBANK1"]["quality"] == [
-        "quality1",
-        "quality2",
-    ]
-    assert "quality" not in node_data.biobanks.rows_by_id["bbmri-eric:ID:NO_Janus"]
-    assert node_data.biobanks.rows_by_id["bbmri-eric:ID:NO_CoronaTrondelag"][
+    assert node_data.biobanks.rows_by_id["bbmri-eric:ID:NL_test_quality_biobank1"][
+        "quality"
+    ] == ["quality1", "quality2"]
+    assert (
+        "quality"
+        not in node_data.biobanks.rows_by_id["bbmri-eric:ID:NL_biobank_noQual"]
+    )
+    assert node_data.biobanks.rows_by_id["bbmri-eric:ID:NL_test_quality_biobank2"][
         "quality"
     ] == ["quality3"]
     assert node_data.collections.rows_by_id[
-        "bbmri-eric:ID:NO_bbmri-eric:ID:NO_CancerBiobankOUH:collection"
-        ":all_samples_samples"
+        "bbmri-eric:ID:NL_bb1:collection:test_quality1"
     ]["quality"] == ["quality1"]
     assert (
         "quality"
         not in node_data.collections.rows_by_id[
-            "bbmri-eric:ID:NO_moba:collection:all_samples"
+            "bbmri-eric:ID:NL_bb1:collection:test_noQual"
         ]
     )
 
 
-def test_transformer_replace_eu_rows_skip_eu():
+def test_transformer_replace_eu_rows_skip_eu(transformer):
     eu = Node("EU", "Europe")
-
     node_data = MagicMock()
     node_data.node = eu
-    transformer = Transformer(
-        node_data=node_data,
-        quality=MagicMock(),
-        printer=Printer(),
-        existing_biobanks=MagicMock(),
-        eu_node_data=MagicMock(),
-    )
+    transformer.node_data = node_data
     transformer._replace_rows = MagicMock()
 
     transformer._replace_eu_rows()
@@ -119,7 +111,7 @@ def test_transformer_replace_eu_rows_skip_eu():
     transformer._replace_rows.assert_not_called()
 
 
-def test_transformer_replace_eu_rows():
+def test_transformer_replace_eu_rows(transformer):
     cy = Node("CY", "Cyprus")
     eu = Node("EU", "Europe")
 
@@ -149,14 +141,9 @@ def test_transformer_replace_eu_rows():
 
     node_data.node = cy
     eu_node_data.node = eu
+    transformer.node_data = node_data
+    transformer.eu_node_data = eu_node_data
 
-    transformer = Transformer(
-        node_data=node_data,
-        quality=MagicMock(),
-        printer=Printer(),
-        existing_biobanks=MagicMock(),
-        eu_node_data=eu_node_data,
-    )
     transformer._replace_rows(cy, persons, eu_persons)
 
     assert persons.rows_by_id["bbmri-eric:contactID:EU_person2"]["name"] == "person2"
@@ -168,7 +155,26 @@ def test_transformer_replace_eu_rows():
     ]
 
 
-def test_transformer_create_combined_networks():
+def test_transformer_set_biobank_labels(transformer):
+    node_data = MagicMock()
+    node_data.collections.rows = [
+        {"biobank": "biobank1", "name": "Collections1"},
+        {"biobank": "biobank2", "name": "Collections2"},
+    ]
+    node_data.biobanks.rows_by_id = {
+        "biobank1": {"name": "BIOBANK1"},
+        "biobank2": {"name": ""},
+    }
+    transformer.node_data = node_data
+
+    transformer._set_biobank_labels()
+
+    assert node_data.collections.rows[0]["biobank_label"] == "BIOBANK1"
+
+    assert node_data.collections.rows[1]["biobank_label"] == ""
+
+
+def test_transformer_create_combined_networks(transformer):
     node_data = MagicMock()
     node_data.collections.rows = [
         {"biobank": "biobank1", "network": []},
@@ -186,14 +192,9 @@ def test_transformer_create_combined_networks():
         "biobank5": {"network": []},
         "biobank6": {"network": []},
     }
+    transformer.node_data = node_data
 
-    Transformer(
-        node_data=node_data,
-        quality=MagicMock(),
-        printer=Printer(),
-        existing_biobanks=MagicMock(),
-        eu_node_data=MagicMock(),
-    )._set_combined_networks()
+    transformer._set_combined_networks()
 
     assert set(node_data.collections.rows[0]["combined_network"]) == {
         "network1",
@@ -210,3 +211,65 @@ def test_transformer_create_combined_networks():
         "network2",
     }
     assert set(node_data.collections.rows[5]["combined_network"]) == set()
+
+
+def test_transformer_combined_quality(node_data, transformer):
+    q_info = QualityInfo(
+        biobanks={},
+        biobank_levels={
+            "bbmri-eric:ID:NL_test_quality_biobank1": ["level_bio1", "level_bio2"],
+            "bbmri-eric:ID:NL_test_quality_biobank2": ["level_bio3"],
+            "bbmri-eric:ID:NL_test_quality_biobank3": ["level_bio_col"],
+        },
+        collections={},
+        collection_levels={
+            "bbmri-eric:ID:NL_bb1:collection:test_quality1": ["level_col5"],
+            "bbmri-eric:ID:NL_bb1:collection:test_quality2": ["level_col2"],
+            "bbmri-eric:ID:NL_bb3:collection:test_quality3": ["level_bio_col"],
+        },
+    )
+    transformer.node_data = node_data
+    transformer.quality = q_info
+
+    transformer._set_combined_qualities()
+
+    assert (
+        "combined_quality"
+        not in node_data.biobanks.rows_by_id["bbmri-eric:ID:NL_biobank_noQual"]
+    )
+    assert sorted(
+        node_data.collections.rows_by_id["bbmri-eric:ID:NL_bb1:collection:test_noQual"][
+            "combined_quality"
+        ]
+    ) == sorted(["level_bio1", "level_bio2"])
+    assert sorted(
+        node_data.collections.rows_by_id[
+            "bbmri-eric:ID:NL_bb1:collection:test_quality2"
+        ]["combined_quality"]
+    ) == sorted(["level_col2", "level_bio3"])
+    assert node_data.collections.rows_by_id[
+        "bbmri-eric:ID:NL_bb1:collection:test_quality1"
+    ]["combined_quality"] == ["level_col5"]
+    assert node_data.collections.rows_by_id[
+        "bbmri-eric:ID:NL_bb3:collection:test_quality3"
+    ]["combined_quality"] == ["level_bio_col"]
+    assert (
+        node_data.collections.rows_by_id["bbmri-eric:ID:NL_valid-collectionID-1"][
+            "combined_quality"
+        ]
+        == []
+    )
+
+
+def test_map_categories(transformer):
+    node_data = MagicMock()
+    collection1 = MagicMock()
+    collection2 = MagicMock()
+    node_data.collections.rows = [collection1, collection2]
+    category_mapper = MagicMock()
+    transformer.node_data = node_data
+    transformer.category_mapper = category_mapper
+
+    transformer._set_collection_categories()
+
+    category_mapper.map.assert_has_calls([call(collection1), call(collection2)])
